from qgis.PyQt.QtCore import (
  QVariant
  )
from qgis.core import (
  QgsProcessingFeedback,
  QgsProcessingContext
  )

from qgis import processing

from ..alginit.initabstract import initabstract

class mergeabstract(initabstract):
  
  def initMultipleLayers(self, parameters, context:QgsProcessingContext, feedback:QgsProcessingFeedback, layers:list, proc_init:str) -> list:
    
    target_crs = None
    merge_layers = []
    for layer in layers:
      if target_crs is None:
        target_crs = layer.sourceCrs()
      else:
        if target_crs != layer.sourceCrs():
          raise Exception("The target CRS is different")
    

      required_field = [fld not in layer.fields().names() for fld in self.FIELDS_REQUIRED]
      
      # check whether the layer needs to be initialized          
      if layer.crs() != target_crs or any(required_field):
        merge_layers.append(
          processing.run(
            proc_init,
            {
              "INPUT": layer,
              "TARGET_CRS": target_crs,
              "OUTPUT": "TEMPORARY_OUTPUT"
            }
          )["OUTPUT"]
        )
      else:
        merge_layers.append(layer)
    
      self.setFieldsDict(
        parameters, context, feedback, 
        layer.fields(),
        overwrite = False
      )
      
    return merge_layers
      