from qgis.PyQt.QtCore import (
  QT_TRANSLATE_NOOP, QVariant
  )
from qgis.core import (
  QgsProcessing,
  QgsProcessingParameterMultipleLayers,
  QgsProcessingParameterFeatureSink,
  QgsProcessingParameterBoolean,
  QgsProcessingParameterCrs,
  QgsWkbTypes
  )

from .mergeabstract import mergeabstract

class mergeelevationpoint(mergeabstract):
  PARAMETERS = {                  
    "INPUT": {
      "ui_func": QgsProcessingParameterMultipleLayers,
      "ui_args":{
        "description": QT_TRANSLATE_NOOP("mergeelevationpoint","Elevation point layers to merge"),
        "layerType": QgsProcessing.TypeVectorPoint
      }
    },
    "TARGET_CRS": {
      "ui_func": QgsProcessingParameterCrs,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("mergeelevationpoint","Target CRS (Cartesian coordinates)")
      }
    },
    "KEEP_ATTRIBUTES": {
      "ui_func": QgsProcessingParameterBoolean,
      "ui_args":{
        "description" : QT_TRANSLATE_NOOP("mergeelevationpoint","Keep all attributes (or only the required fields)?"),
        "defaultValue": True
      }
    },
    "OUTPUT": {
      "ui_func": QgsProcessingParameterFeatureSink,
      "ui_args": {
        "description": QT_TRANSLATE_NOOP("mergeelevationpoint","Merged elevation points" )
      }
    }
  }
  
  def initAlgorithm(self, config):
    (rect, target_crs) = self.getExtentAndCrsUsingCanvas()
    self.PARAMETERS["TARGET_CRS"]["ui_args"]["defaultValue"] = target_crs.authid()

    self.initParameters()
  
  
  def processAlgorithm(self, parameters, context, feedback):
    target_crs = self.parameterAsCrs(parameters, "TARGET_CRS", context)
    # check whether the target CRS is x-y coordinates
    self.checkCrsAsCartesian(target_crs, feedback)

    # initialize all the layers
    merge_layers = self.initMultipleLayers(
      parameters, context, feedback,
      layers = self.parameterAsLayerList(parameters, "INPUT", context),
      proc_init= "hrisk:initelevationpoint"
    )
    
    # define the feature sink
    (sink, dest_id) = self.parameterAsSink(
      parameters, "OUTPUT", context, 
      self.FIELDS_INIT, 
      QgsWkbTypes.PointZ,
      target_crs
    )
    
    self.resetPk()
    
    # add all the features
    for layer in merge_layers:
      self.addFeaturesToSink(sink, dest_id, layer)
              
    return {"OUTPUT": dest_id}
  
  def createInstance(self):
    return mergeelevationpoint()

  def displayName(self):
    return self.tr("Elevation point")

  def group(self):
    return self.tr("Merge features")

  def groupId(self):
    return "mergefeature"

  def createInstance(self):
    return mergeelevationpoint()
